@@ -37,6 +37,7 @@ typedef struct
 {
     GtkWidget *event_toolbar;
     GtkWidget *event;
+    GtkWidget *info_bar;
 } GlWindowPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GlWindow, gl_window, GTK_TYPE_APPLICATION_WINDOW)
@@ -168,6 +169,95 @@ on_search (GSimpleAction *action,
     g_simple_action_set_state (action, variant);
 }
 
+static void
+on_export (GSimpleAction *action,
+           GVariant *variant,
+           gpointer user_data)
+{
+    GlWindowPrivate *priv;
+    GlEventView *event;
+    GtkFileChooser *file_chooser;
+    GtkWidget *dialog;
+    gint res;
+
+    priv = gl_window_get_instance_private (GL_WINDOW (user_data));
+    event = GL_EVENT_VIEW (priv->event);
+
+    dialog = gtk_file_chooser_dialog_new (_("Save logs"),
+                                          GTK_WINDOW (user_data),
+                                          GTK_FILE_CHOOSER_ACTION_SAVE,
+                                          _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                          _("_Save"), GTK_RESPONSE_ACCEPT,
+                                          NULL);
+
+    file_chooser = GTK_FILE_CHOOSER (dialog);
+    gtk_file_chooser_set_do_overwrite_confirmation (file_chooser, TRUE);
+    gtk_file_chooser_set_current_name (file_chooser, _("log messages"));
+
+    res = gtk_dialog_run (GTK_DIALOG (dialog));
+    if (res == GTK_RESPONSE_ACCEPT)
+    {
+        gboolean have_error = FALSE;
+        gchar *file_content;
+        GFile *output_file;
+        GFileOutputStream *file_ostream;
+        GError *error = NULL;
+        GtkWidget *error_dialog;
+
+        file_content = gl_event_view_get_output_logs (event);
+        output_file = gtk_file_chooser_get_file (file_chooser);
+        file_ostream = g_file_replace (output_file, NULL, TRUE,
+                                       G_FILE_CREATE_NONE, NULL, &error);
+        if (error != NULL)
+        {
+            have_error = TRUE;
+
+            g_warning ("Error while replacing exported log messages file: %s",
+                       error->message);
+            g_clear_error (&error);
+        }
+
+        g_output_stream_write (G_OUTPUT_STREAM (file_ostream), file_content,
+                               strlen (file_content), NULL, &error);
+        if (error != NULL)
+        {
+            have_error = TRUE;
+
+            g_warning ("Error while replacing exported log messages file: %s",
+                       error->message);
+            g_clear_error (&error);
+        }
+
+        g_output_stream_close (G_OUTPUT_STREAM (file_ostream), NULL, &error);
+        if (error != NULL)
+        {
+            have_error = TRUE;
+
+            g_warning ("Error while replacing exported log messages file: %s",
+                       error->message);
+            g_clear_error (&error);
+        }
+
+        if (have_error == TRUE)
+        {
+            error_dialog = gtk_message_dialog_new (GTK_WINDOW (user_data),
+                                                   GTK_DIALOG_MODAL,
+                                                   GTK_MESSAGE_ERROR,
+                                                   GTK_BUTTONS_CLOSE,
+                                                   "%s",
+                                                   _("Unable to export log messages to a file"));
+            gtk_dialog_run (GTK_DIALOG (error_dialog));
+            gtk_widget_destroy (error_dialog);
+        }
+
+        g_free (file_content);
+        g_object_unref (file_ostream);
+        g_object_unref (output_file);
+    }
+
+    gtk_widget_destroy (dialog);
+}
+
 static void
 on_view_boot (GSimpleAction *action,
               GVariant *variant,
@@ -255,6 +345,30 @@ on_gl_window_key_press_event (GlWindow *window,
     return GDK_EVENT_PROPAGATE;
 }
 
+static void
+on_help_button_clicked (GlWindow *window,
+                        gint response_id,
+                        gpointer user_data)
+{
+    GlWindowPrivate *priv;
+    GtkWindow *parent;
+    GError *error = NULL;
+
+    parent = GTK_WINDOW (window);
+    priv = gl_window_get_instance_private (GL_WINDOW (window));
+
+    gtk_show_uri (gtk_window_get_screen (parent), "help:gnome-logs/permissions",
+                  GDK_CURRENT_TIME, &error);
+
+    if (error)
+    {
+        g_debug ("Error while opening help: %s", error->message);
+        g_error_free (error);
+    }
+
+    gtk_widget_hide (priv->info_bar);
+}
+
 void
 gl_window_set_sort_order (GlWindow *window,
                           GlSortOrder sort_order)
@@ -275,6 +389,7 @@ static GActionEntry actions[] = {
     { "toolbar-mode", on_action_radio, "s", "'list'", on_toolbar_mode },
     { "search", on_action_toggle, NULL, "false", on_search },
     { "view-boot", on_action_radio, "s", "''", on_view_boot },
+    { "export", on_export },
     { "close", on_close }
 };
 
@@ -289,9 +404,13 @@ gl_window_class_init (GlWindowClass *klass)
                                                   event_toolbar);
     gtk_widget_class_bind_template_child_private (widget_class, GlWindow,
                                                   event);
+    gtk_widget_class_bind_template_child_private (widget_class, GlWindow,
+                                                  info_bar);
 
     gtk_widget_class_bind_template_callback (widget_class,
                                              on_gl_window_key_press_event);
+    gtk_widget_class_bind_template_callback (widget_class,
+                                             on_help_button_clicked);
 }
 
 static void
@@ -340,6 +459,34 @@ gl_window_init (GlWindow *window)
                                                GTK_STYLE_PROVIDER (provider),
                                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
+    if (!gl_util_can_read_system_journal ())
+    {
+        GtkWidget *message_label;
+        GtkWidget *content_area;
+
+        message_label = gtk_label_new (_("Unable to read system logs"));
+        gtk_widget_set_hexpand (GTK_WIDGET (message_label), TRUE);
+        gtk_widget_show (message_label);
+        content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (priv->info_bar));
+        gtk_container_add (GTK_CONTAINER (content_area), message_label);
+
+        gtk_widget_show (priv->info_bar);
+    }
+
+    if (!gl_util_can_read_user_journal ())
+    {
+        GtkWidget *message_label;
+        GtkWidget *content_area;
+
+        message_label = gtk_label_new (_("Unable to read user logs"));
+        gtk_widget_set_hexpand (GTK_WIDGET (message_label), TRUE);
+        gtk_widget_show (message_label);
+        content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (priv->info_bar));
+        gtk_container_add (GTK_CONTAINER (content_area), message_label);
+
+        gtk_widget_show (priv->info_bar);
+    }
+
     g_object_unref (provider);
 }
 
